egison-5.0.0: Programming language with non-linear pattern-matching against non-free data
LicenseMIT
Safe HaskellNone
LanguageGHC2021

Language.Egison

Description

This is the top module of Egison.

Synopsis

Documentation

Modules needed to execute Egison

Environment

initialEnv :: EvalM Env Source #

Create initial environment with only primitive functions Core libraries will be loaded separately to maintain consistent Env chain Returns EvalM Env to preserve EvalState (type environment, class environment)

Information

version :: Version Source #

Version number