egison-5.0.0: Programming language with non-linear pattern-matching against non-free data
LicenseMIT
Safe HaskellNone
LanguageGHC2021

Language.Egison.VarEntry

Description

This module defines the VarEntry data structure used by both evaluation environment (Data.hs) and type inference environment (Type/Env.hs).

Synopsis

Documentation

data VarEntry a Source #

Variable entry in the environment Contains indices and value for a single variable binding Parametrized to support both ObjectRef (for evaluation) and TypeScheme (for type inference)

Constructors

VarEntry 

Fields

Instances

Instances details
Show a => Show (VarEntry a) Source # 
Instance details

Defined in Language.Egison.VarEntry

Methods

showsPrec :: Int -> VarEntry a -> ShowS #

show :: VarEntry a -> String #

showList :: [VarEntry a] -> ShowS #

Eq a => Eq (VarEntry a) Source # 
Instance details

Defined in Language.Egison.VarEntry

Methods

(==) :: VarEntry a -> VarEntry a -> Bool #

(/=) :: VarEntry a -> VarEntry a -> Bool #