| License | MIT |
|---|---|
| Safe Haskell | None |
| Language | GHC2021 |
Language.Egison.VarEntry
Description
This module defines the VarEntry data structure used by both evaluation environment (Data.hs) and type inference environment (Type/Env.hs).
| License | MIT |
|---|---|
| Safe Haskell | None |
| Language | GHC2021 |
Language.Egison.VarEntry
Description
This module defines the VarEntry data structure used by both evaluation environment (Data.hs) and type inference environment (Type/Env.hs).